var maik = (function (exports) {
'use strict';

const SVG_NAMESPACE = 'http://www.w3.org/2000/svg';

const slice = [].slice;

// just a minifier friendly indirection
function createDocumentFragment(document) {
  return document.createDocumentFragment()
}

// given a node, it does what is says
function createText(node, text) {
  return node.ownerDocument.createTextNode(text)
}

const featureFragment = createDocumentFragment(document);

// use native .append(...childNodes) where available
const appendNodes =
  'append' in featureFragment
    ? function appendNodes(node, childNodes) {
        node.append.apply(node, childNodes);
      }
    : function appendNodes(node, childNodes) {
        for (var i = 0, length = childNodes.length; i < length; i++) {
          node.appendChild(childNodes[i]);
        }
      };

const HAS_IMPORT_NODE = 'importNode' in document;

// Fix for IE
const REMOVES_EMPTY_TEXTNODES_ON_CLONE = (function() {
  featureFragment.appendChild(createText(featureFragment, 'g'));
  featureFragment.appendChild(createText(featureFragment, ''));
  let clone = HAS_IMPORT_NODE
    ? document.importNode(featureFragment, true)
    : featureFragment.cloneNode(true);
  return clone.childNodes.length === 1
})();

const cloneNode = REMOVES_EMPTY_TEXTNODES_ON_CLONE
  ? function cloneNode(node) {
      let clone = node.cloneNode();
      let childNodes = node.childNodes || [];
      for (var i = 0, l = childNodes.length; i < l; i++) {
        clone.appendChild(cloneNode(childNodes[i]));
      }
      return clone
    }
  : node => node.cloneNode(true);

const importNode =
  HAS_IMPORT_NODE && !REMOVES_EMPTY_TEXTNODES_ON_CLONE
    ? (document, node) => document.importNode(node, true)
    : (_, node) => cloneNode(node);

const NODE_FILTER =
  NodeFilter.SHOW_ELEMENT | NodeFilter.SHOW_COMMENT | NodeFilter.SHOW_TEXT;

const createTreeWalker = node =>
  node.ownerDocument.createTreeWalker(node, NODE_FILTER, null, false);

// TODO: `createFragment` has some uniqueness, maybe put it somewhere else?

// given a node, inject some html and return
// the resulting template document fragment
function createFragment(document, svg, html) {
  return (svg ? createSVGFragment : createHTMLFragment)(document, html)
}

const SUPPORTS_TEMPLATE = 'content' in document.createElement('template');

// create fragment for HTML
const createHTMLFragment = SUPPORTS_TEMPLATE
  ? function createHTMLFragment(document, html) {
      let container = document.createElement('template');
      container.innerHTML = html;
      return container.content
    }
  : function createHTMLFragmentLegacy(document, html) {
      let fragment = createDocumentFragment(document);
      let container = document.createElement('div');

      // el.innerHTML = '<td></td>'; is not possible
      // if the content is a partial internal table content
      // it needs to be wrapped around once injected.
      // HTMLTemplateElement does not suffer this issue.
      let needsTableWrap = /^[^\S]*?<(col(?:group)?|t(?:head|body|foot|r|d|h))/i.test(
        html
      );

      if (needsTableWrap) {
        // secure the RegExp.$1 result ASAP to avoid issues
        // in case a non-browser DOM library uses RegExp internally
        // when HTML content is injected (basicHTML / jsdom / others...)
        let selector = RegExp.$1;
        container.innerHTML = '<table>' + html + '</table>';
        appendNodes(
          fragment,
          slice.call(container.querySelectorAll(selector))
        );
      } else {
        container.innerHTML = html;
        appendNodes(fragment, slice.call(container.childNodes));
      }

      return fragment
    };

// create a fragment for SVG
const createSVGFragment = SUPPORTS_TEMPLATE
  ? function createSVGFragment(document, html) {
      let container = document.createElement('template');
      let svgElement = document.createElementNS(SVG_NAMESPACE, 'svg');
      svgElement.innerHTML = html;
      appendNodes(container.content, slice.call(svgElement.childNodes));
      return container.content
    }
  : function createSVGFragmentLegacy(document, html) {
      var fragment = createDocumentFragment(document);
      let container = document.createElement('div');
      container.innerHTML =
        '<svg xmlns="' + SVG_NAMESPACE + '">' + html + '</svg>';
      appendNodes(fragment, slice.call(container.firstChild.childNodes));
      return fragment
    };

const EXPANDO = '_hyper_';
const UID = EXPANDO + ((Math.random() * new Date()) | 0) + ';';
const UIDC = '<!--' + UID + '-->';

const ELEMENT_NODE = 1;
const ATTRIBUTE_NODE = 2;
const TEXT_NODE = 3;
const COMMENT_NODE = 8;

// eslint-disable-next-line no-control-regex
const EXTRACT_ATTRIBUTE_NAME = /\s([^\0-\x1F\x7F-\x9F \x09\x0a\x0c\x0d"'>=/]+)[ \x09\x0a\x0c\x0d]*=[ \x09\x0a\x0c\x0d]*['"]?$/;
const WHITESPACE_NODE_TEST = /^\n\s*$/;

/*
  Given the unique static strings of a template-tag invocation,
  create a blueprint fragment and notes about its dynamic parts
  which we can use over and over for new instances of this (template)
  fragment.
 */
function createTemplateBlueprint$1(
  nodeCallback,
  attributeCallback,
  strings,
  document,
  isSvg
) {
  let { html, notes } = processStrings(
    strings,
    nodeCallback,
    attributeCallback
  );
  let fragment = createFragment(document, isSvg, html);
  return processFragment(notes, fragment)
}

/*
  The functions `findTagClose` and `processStrings` originally come from the
  lit-html PR https://github.com/PolymerLabs/lit-html/pull/153
  authored by https://github.com/jridgewell

  Slightly modified to actually work.
 */

/*
  Finds the closing index of the last closed HTML tag.
  This has 3 possible return values:
    - `-1`, meaning there is no tag in str.
    - `string.length`, meaning the last opened tag is unclosed.
    - Some positive number < str.length, meaning the index of the closing '>'.
 */
function findTagClose(str) {
  const close = str.lastIndexOf('>');
  const open = str.indexOf('<', close + 1);
  return open > -1 ? str.length : close
}

// In general, walk the static strings the user gave us, and decide which
// 'hole'/interpolation will be on an attribute or is in a node position.
// Prepare the HTML we will actually use as a template. Return the HTML
// and notes about each 'hole' we found.
const processStrings = (
  strings,
  nodeCallback,
  attributeCallback,
  nodeMarker = UIDC,
  attributeMarker = UID
) => {
  const last = strings.length - 1;
  let html = '';
  let notes = [];

  let isTextBinding = true;
  for (let i = 0; i < last; i++) {
    const s = strings[i];
    html += s;
    // We're in a text position if the previous string closed its tags.
    // If it doesn't have any tags, then we use the previous text position
    // state.
    const closing = findTagClose(s);
    isTextBinding = closing > -1 ? closing < s.length : isTextBinding;
    if (isTextBinding) {
      html += nodeMarker;
      notes.push({ type: ELEMENT_NODE, updater: nodeCallback });
    } else {
      html += attributeMarker;
      // In general attributes are stored in `node.attributes` as a map, which
      // means they're unordered. But user's `strings` is ordered and we MUST
      // retain this order. We therefore extract the wanted attributeName from
      // the current string (`s`).
      let name = extractAttributeName(s);
      notes.push({ type: ATTRIBUTE_NODE, updater: attributeCallback(name) });
    }
  }
  html += strings[last];
  return { html, notes }
};

const extractAttributeName = string => {
  let match = EXTRACT_ATTRIBUTE_NAME.exec(string);
  if (!match) {
    throw new Error(
      `Could not get the attribute name within the following String '${
        string
      }'`
    )
  }

  return match[1]
};

const processFragment = (
  firstNotes,
  fragment,
  nodeMarker = UIDC,
  marker = UID
) => {
  let notes = [];
  let nextNode = domWalker(fragment, nodeMarker, marker);
  // We keep an indexOffset here in case we create text nodes the walker did
  // not count.
  let indexOffset = 0;

  // We HAVE to remove each attribute we will find at the end of this fn,
  // otherwise the user would see our markers.
  let foundAttributes = [];

  for (let i = 0, l = firstNotes.length; i < l; i++) {
    let earlyNote = firstNotes[i];
    let { type: typeHint, updater } = earlyNote;

    let [node, index] = nextNode(typeHint);
    switch (node.nodeType) {
      case ATTRIBUTE_NODE:
        notes.push({ index: index + indexOffset, updater });
        foundAttributes.push(node);
        break
      case COMMENT_NODE:
        notes.push({ index: index + indexOffset, updater });
        break
      case TEXT_NODE: {
        // Whenever we encounter a TEXT_NODE, the browser actually failed at
        // inserting a COMMENT_NODE, because that's what we asked for. This
        // is the case e.g. for <style> tags, which can only have TEXT_NODEs
        // as children.

        // Now let the input be html`<style>a${}b${}c</style>`. The walker
        // sees exactly ONE text node (with {{~}} denoting our marker):
        //   `a{{~}}b${{~}}c`
        // We have to fulfill TWO notes and since the node updater always
        // controls and fills the space BEFORE him, we somehow must end up
        // with three text nodes 'a', 'b' and 'c'; and at the same time push
        // two notes referring to 'b' and 'c'.

        let document = node.ownerDocument;
        let parentNode = node.parentNode;

        // We split the string into its static parts.
        let value = node.nodeValue;
        let parts = value.split(nodeMarker);

        // For all parts except the last one, we create a new text node, and
        // a note
        let last = parts.length - 1;
        for (let j = 0; j < last; j++) {
          let textNode = document.createTextNode(parts[j]);
          // `insertBefore` to not move the position of the domWalker
          parentNode.insertBefore(textNode, node);

          // Create a note pointing to the *next* node
          indexOffset++;
          notes.push({
            index: index + indexOffset,
            updater: earlyNote.updater
          });
          // ATT: We MUST forward the outer for-loop
          earlyNote = firstNotes[++i];
        }

        // Reuse the existing node for the last part.
        node.nodeValue = parts[last];
      }
    }
  }

  // Completely walk the DOM, and remove some whitespace nodes.
  nextNode();

  for (let i = 0, l = foundAttributes.length; i < l; i++) {
    let attribute = foundAttributes[i];
    let node = attribute.ownerElement;
    node.removeAttributeNode(attribute);
  }

  return { fragment, notes }
};

// This is like a normal `TreeWalker` but yields only nodes with markers.
// The returned `next` function accepts a `typeHint` for optimization purposes.
// Say, we expect the next node to be a 'node', we can just skip processing
// attributes.
const domWalker = (node, nodeMarker, marker) => {
  let walker = createTreeWalker(node);
  let index = -1;

  let attributes = null;
  let attrIndex = 0;

  let whitespaceOnlyNodes = [];

  return function next(typeHint) {
    while (true) {
      if (attributes && typeHint === ATTRIBUTE_NODE) {
        for (let l = attributes.length; attrIndex < l; attrIndex++) {
          let attr = attributes[attrIndex];
          if (attr.value === marker) {
            // ATT: We MUST increment the for-loop var manually before
            // returning
            attrIndex++;
            return [attr, index]
          }
        }

        attributes = null;
      }

      WALKER: {
        while (walker.nextNode()) {
          index++;
          let node = walker.currentNode;

          switch (node.nodeType) {
            case ELEMENT_NODE:
              if (typeHint === ATTRIBUTE_NODE && node.hasAttributes()) {
                attributes = node.attributes;
                attrIndex = 0;
                break WALKER
              }
              continue
            case COMMENT_NODE:
              if (node.nodeValue === marker) {
                return [node, index]
              }
              continue
            case TEXT_NODE: {
              let nodeValue = node.nodeValue;
              if (WHITESPACE_NODE_TEST.test(nodeValue)) {
                index--;
                whitespaceOnlyNodes.push(node);
              } else if (nodeValue.indexOf(nodeMarker) !== -1) {
                return [node, index]
              }
              continue
            }
          }
        }

        for (let i = 0, l = whitespaceOnlyNodes.length; i < l; i++) {
          let node = whitespaceOnlyNodes[i];
          node.parentNode.removeChild(node);
        }
        return
      }
    }
  }
};

const createTemplateInstance = (
  createTemplateBlueprint,
  document,
  isSvg,
  strings
) => {
  let { fragment, notes } = createTemplateBlueprint(strings, document, isSvg);
  return instantiateBlueprint(document, fragment, notes)
};

const instantiateBlueprint = (document, blueprintFragment, notes) => {
  let fragment = importNode(document, blueprintFragment);
  let updaters = readyUpdaters(fragment, notes);
  return { fragment, updaters }
};

// Find the correct live node using our notes, and pass it to the
// updater fns. Return just the updaters.
function readyUpdaters(fragment, notes) {
  let walker = createTreeWalker(fragment);
  let index = -1;
  let updates = [];
  for (let i = 0, l = notes.length; i < l; i++) {
    let { updater, index: nodeIndex } = notes[i];

    while (index < nodeIndex) {
      index++;
      walker.nextNode();
    }

    updates[i] = updater(walker.currentNode);
  }
  return updates
}

/* Pseudo Polyfill. Just enough to get it working. */

// Map with partial double Array fallback
const Map =
  self.Map ||
  function() {
    var k = [],
      v = [];
    return {
      get: function(obj) {
        return v[k.indexOf(obj)]
      },
      // being used with unique template literals
      // there is never a case when a value is overwritten
      // no need to check upfront for the indexOf
      set: function(obj, value) {
        v[k.push(obj) - 1] = value;
      }
    }
  };

const memoizeOnFirstArg = fn => {
  let cache = new Map();
  return (arg, ...args) => {
    let rv = cache.get(arg);
    if (rv === undefined) {
      rv = fn(arg, ...args);
      cache.set(arg, rv);
    }
    return rv
  }
};

const lruCacheOne = fn => {
  let lastIn, lastOut;
  return (arg, ...args) => {
    if (lastIn !== arg) {
      lastIn = arg;
      lastOut = fn(arg, ...args);
    }
    return lastOut
  }
};

const flatten = array => [].concat(...array);

// used to convert childNodes to Array
const slice$1 = [].slice;

// older WebKit need this


// BB7 and webOS need this
const isArray =
  Array.isArray ||
  (function() {
    var toString = {}.toString;
    // I once had an engine returning [array Array]
    // and I've got scared since!
    var s = toString.call([]);
    return function(a) {
      return toString.call(a) === s
    }
  })();

const isObject = x => x != null && typeof x === 'object';

// Keep it stupid simple here
const isObservable = value => isObject(value) && 'subscribe' in value;
const isSubject = value => isObject(value) && 'next' in value;

const repeat = (seq, mapFn) => {
  if (isObservable(seq)) {
    return seq.map(items => items.map(mapFn))
  }
  return seq.map(mapFn)
};

// entry point for all TL => DOM operations
function render(
  createTemplateInstance,
  finalSideEffect,
  strings,
  ...values
) {
  strings = unique(strings);
  let { fragment, updaters } = createTemplateInstance(strings);
  update(updaters, values);
  return finalSideEffect(fragment)
}

// invokes each update function passing interpolated value
function update(updateFns, values) {
  for (var i = 0, length = updateFns.length; i < length; i++) {
    updateFns[i](values[i]);
  }
}

const replaceNodeContent = (node, fragment) => {
  node.textContent = '';
  node.appendChild(fragment);
  return node
};

// return a single node or an Array or nodes
const extractContent = node => {
  let childNodes = node.childNodes;
  return childNodes.length === 1 ? childNodes[0] : slice$1.call(childNodes)
};

// Both Firefox < 55 and TypeScript have issues with template literals.
// This lazy defined callback should spot issues right away
// and in the best case scenario become a no-op
let unique = strings => {
  // ATT: If we inline `fn`, uglify changes the behavior of our code!
  const fn = strings => strings;
  const testFn = () => fn``;
  if (testFn() === testFn()) {
    unique = strings => strings;
  } else {
    const templateObjects = Object.create(null);
    unique = strings => {
      let key = strings.join(UIDC);
      return templateObjects[key] || (templateObjects[key] = strings)
    };
  }
  return unique(strings)
};

/*! (c) 2017 Andrea Giammarchi (ISC) */

/**
 * This code is a revisited port of the snabbdom vDOM diffing logic,
 * the same that fuels as fork Vue.js or other libraries.
 * @credits https://github.com/snabbdom/snabbdom
 */

const identity = O => O;

const domdiff = (
  parentNode,     // where changes happen
  currentNodes,   // Array of current items/nodes
  futureNodes,    // Array of future items/nodes
  getNode,        // optional way to retrieve a node from an item
  beforeNode      // optional item/node to use as insertBefore delimiter
) => {
  const get = getNode || identity;
  const before = beforeNode == null ? null : get(beforeNode, 0);
  let currentStart = 0, futureStart = 0;
  let currentEnd = currentNodes.length - 1;
  let currentStartNode = currentNodes[0];
  let currentEndNode = currentNodes[currentEnd];
  let futureEnd = futureNodes.length - 1;
  let futureStartNode = futureNodes[0];
  let futureEndNode = futureNodes[futureEnd];
  while (currentStart <= currentEnd && futureStart <= futureEnd) {
    if (currentStartNode == null) {
      currentStartNode = currentNodes[++currentStart];
    }
    else if (currentEndNode == null) {
      currentEndNode = currentNodes[--currentEnd];
    }
    else if (futureStartNode == null) {
      futureStartNode = futureNodes[++futureStart];
    }
    else if (futureEndNode == null) {
      futureEndNode = futureNodes[--futureEnd];
    }
    else if (currentStartNode == futureStartNode) {
      currentStartNode = currentNodes[++currentStart];
      futureStartNode = futureNodes[++futureStart];
    }
    else if (currentEndNode == futureEndNode) {
      currentEndNode = currentNodes[--currentEnd];
      futureEndNode = futureNodes[--futureEnd];
    }
    else if (currentStartNode == futureEndNode) {
      parentNode.insertBefore(
        get(currentStartNode, 1),
        get(currentEndNode, -0).nextSibling
      );
      currentStartNode = currentNodes[++currentStart];
      futureEndNode = futureNodes[--futureEnd];
    }
    else if (currentEndNode == futureStartNode) {
      parentNode.insertBefore(
        get(currentEndNode, 1),
        get(currentStartNode, 0)
      );
      currentEndNode = currentNodes[--currentEnd];
      futureStartNode = futureNodes[++futureStart];
    }
    else {
      let index = currentNodes.indexOf(futureStartNode);
      if (index < 0) {
        parentNode.insertBefore(
          get(futureStartNode, 1),
          get(currentStartNode, 0)
        );
        futureStartNode = futureNodes[++futureStart];
      }
      else {
        let el = currentNodes[index];
        currentNodes[index] = null;
        parentNode.insertBefore(
          get(el, 1),
          get(currentStartNode, 0)
        );
        futureStartNode = futureNodes[++futureStart];
      }
    }
  }
  if (currentStart > currentEnd) {
    const pin = futureNodes[futureEnd + 1];
    const place = pin != null ? get(pin, 0) : before;
    while (futureStart <= futureEnd) {
      const ch = futureNodes[futureStart++];
      // ignore until I am sure the else could never happen.
      // it might be a vDOM thing 'cause it never happens here.
      /* istanbul ignore else */
      if (ch != null) parentNode.insertBefore(get(ch, 1), place);
    }
  }
  // ignore until I am sure the else could never happen.
  // it might be a vDOM thing 'cause it never happens here.
  /* istanbul ignore else */
  else if (futureStart > futureEnd) {
    while (currentStart <= currentEnd) {
      const ch = currentNodes[currentStart++];
      if (ch != null) parentNode.removeChild(get(ch, -1));
    }
  }
  return futureNodes;
};

function TagInvocation(strings, values, isSvg, key) {
  this.strings = strings;
  this.values = values;
  this.isSvg = isSvg;
  this.key = key;
}

const TEXT_NODE$1 = 3;
const DOCUMENT_FRAGMENT_NODE = 11;

// transformers registry; shared and global, yeah!
var transformers = {};
var transformersKeys = [];

class NodeHolder {
  constructor(marker, managedNodes = []) {
    this._marker = marker;
    this._managedNodes = managedNodes;
  }

  _optimist(nextNodes) {
    this._managedNodes = domdiff(
      this._marker.parentNode,
      this._managedNodes,
      nextNodes,
      null,
      this._marker
    );
  }

  setText(value) {
    let managedNodes = this._managedNodes;
    let length = managedNodes.length;
    if (length === 1 && managedNodes[0].nodeType === TEXT_NODE$1) {
      // Reuse existing TEXT_NODE
      managedNodes[0].nodeValue = value;
    } else {
      if (length) {
        // Clear everything first, to optimize for the following append
        this.clear();
      }
      this._optimist([createText(this._marker, value)]);
    }
  }

  setHTML(value) {
    let fragment = createFragment(this._marker.ownerDocument, false, value);
    this._optimist(slice$1.call(fragment.childNodes));
  }

  setNodes(value) {
    this._optimist(
      value.nodeType === DOCUMENT_FRAGMENT_NODE
        ? slice$1.call(value.childNodes)
        : isArray(value) ? value : [value]
    );
  }

  appendNodes(value) {
    let nodes = this._managedNodes.concat(value);
    this._optimist(nodes);
  }

  appendText(value) {
    this.appendNodes([createText(this._marker, value)]);
  }

  appendHTML(value) {
    let fragment = createFragment(this._marker.ownerDocument, false, value);
    this.appendNodes(slice$1.call(fragment.childNodes));
  }

  clear() {
    this._optimist([]);
  }
}

const makeNodeSetter = nodeMarker => {
  let holder = new NodeHolder(nodeMarker);

  let oldValue;
  let wires = Object.create(null);

  return function setValue(value) {
    if (value === oldValue) {
      return
    }
    oldValue = value;

    if (value == null) {
      value = [];
    }

    switch (typeof value) {
      case 'string':
      case 'number':
      case 'boolean':
        holder.setText(value);
        return
    }

    if (isNode(value)) {
      holder.setNodes(value);
      return
    }

    if (value instanceof TagInvocation) {
      let tagInvocation = value;
      let key = tagInvocation.key || tagInvocation.type;
      let wire$$1 = wires[key] || (wires[key] = materializer());
      holder.setNodes(wire$$1(tagInvocation));
      return
    }

    if (
      isArray(value) ||
      ('length' in value && (value = slice$1.call(value)))
    ) {
      let length = value.length;

      if (length === 0) {
        wires = Object.create(null);
        holder.clear();
        return
      }

      switch (typeof value[0]) {
        case 'string':
        case 'number':
        case 'boolean':
          holder.setText(value.join(''));
          return
        case 'object':
          if (isArray(value[0])) {
            setValue(flatten(value));
            return
          }
          if (isPromise(value[0])) {
            Promise.all(value).then(setValue);
            return
          }
          if (value[0] instanceof TagInvocation) {
            let newWires = Object.create(null);
            for (let i = 0; i < length; i++) {
              let tagInvocation = value[i];
              let key = tagInvocation.key || i;
              let wire$$1 = (newWires[key] = wires[key] || materializer());
              value[i] = wire$$1(tagInvocation);
            }
            wires = newWires;
            value = flatten(value);
          }
      }

      holder.setNodes(value);
      return
    }

    if (isPromise(value)) {
      value.then(setValue);
    } else if ('html' in value) {
      holder.setHTML(value.html);
    } else {
      invokeTransformer(value, setValue, holder);
    }
  }
};

// last attempt to transform content
function invokeTransformer(value, setValue, nodeHolder) {
  for (var key, i = 0, length = transformersKeys.length; i < length; i++) {
    key = transformersKeys[i];
    if (value.hasOwnProperty(key)) {
      transformers[key](value[key], setValue, nodeHolder, value);
    }
  }
}

// quick and dirty Node check
function isNode(value) {
  return 'ELEMENT_NODE' in value
}

// quick and dirty Promise check
function isPromise(value) {
  return 'then' in value
}

const makeAttributeSetter = (node, attributeName) => {
  let oldValue;

  return value => {
    if (value === true) {
      value = '';
    }

    if (oldValue === value) {
      return
    }

    oldValue = value;

    if (value == null || value === false) {
      node.removeAttribute(attributeName);
    } else {
      node.setAttribute(attributeName, value);
    }
  }
};

const makePropertySetter = (node, propertyName) => {
  if (propertyName in node) {
    return makePropertySetterWithDirtyCheck(node, propertyName)
  }
  return makeSimplePropertySetter(node, propertyName)
};

const makeSimplePropertySetter = (node, propertyName) => {
  return value => {
    node[propertyName] = value;
  }
};

const makePropertySetterWithDirtyCheck = (node, propertyName) => {
  let oldValue = null;

  return value => {
    let valueIsPrimitive = typeof value != 'object' || value === null;
    if (value == null) {
      value = '';
    }

    if (
      valueIsPrimitive &&
      oldValue === value &&
      node[propertyName] === value
    ) {
      return
    }

    oldValue = value;

    node[propertyName] = value;
  }
};

const makeEventHandler = (node, eventName) => {
  let currentValue;
  let handler = ev => currentValue(ev);

  return nextValue => {
    if (currentValue === nextValue) {
      return
    }

    if (!nextValue) {
      node.removeEventListener(eventName, handler, false);
    } else if (!currentValue) {
      node.addEventListener(eventName, handler, false);
    }

    currentValue = nextValue;
  }
};

const rxAware = fn => (...args) => {
  let oldValue, subscription;
  let sink = fn(...args);

  return value => {
    if (isObservable(value)) {
      if (oldValue === value) {
        return
      }
      oldValue = value;

      if (subscription) {
        subscription.unsubscribe();
        subscription = null;
      }
      if (value) {
        subscription = value.subscribe(sink);
      }
    } else {
      oldValue = value;

      if (subscription) {
        subscription.unsubscribe();
        subscription = null;
      }

      sink(value);
    }
  }
};

const makeRxAttributeSetter = rxAware(makeAttributeSetter);
const makeRxPropertySetter = rxAware(makePropertySetter);

const makeRxEventHandler = (node, eventName) => {
  let sink = makeEventHandler(node, eventName);

  return function setter(value) {
    if (isSubject(value)) {
      sink(ev => value.next(ev));
    } else {
      sink(value);
    }
  }
};

const attributeCallback = (
  attributeName,
  {
    makeAttributeSetterF = makeAttributeSetter,
    makePropertySetterF = makePropertySetter,
    makeEventHandlerF = makeEventHandler
  } = {}
) => {
  if (attributeName.startsWith('on')) {
    // Allow `onclick` as well as `on-click`
    let eventName =
      attributeName[2] === '-'
        ? attributeName.slice(3)
        : attributeName.slice(2);
    eventName = eventName.toLowerCase();
    return node => makeEventHandlerF(node, eventName)
  }

  if (attributeName.endsWith('$')) {
    let newName = attributeName.slice(0, -1);
    return node => {
      return makeAttributeSetterF(node, newName)
    }
  }

  return node => makePropertySetterF(node, attributeName)
};

const rwAwareNodeCallback = rxAware(makeNodeSetter);

const rxAwareAttributeCallback = name => {
  return attributeCallback(name, {
    makeAttributeSetterF: makeRxAttributeSetter,
    makePropertySetterF: makeRxPropertySetter,
    makeEventHandlerF: makeRxEventHandler
  })
};

/*! (c) 2017 Andrea Giammarchi @WebReflection, (ISC) */
/*! (c) 2017 herr.kaste, (ISC) */

const OWNER_SVG_ELEMENT = 'ownerSVGElement';

const createTemplateBlueprint$$1 = memoizeOnFirstArg(
  createTemplateBlueprint$1.bind(
    null,
    rwAwareNodeCallback,
    rxAwareAttributeCallback
  )
);

const upgrade = createTemplateInstance.bind(null, createTemplateBlueprint$$1);

// The main render factories

// render TL inside a DOM node used as context
const bind = node => {
  let finalSideEffect = lruCacheOne(replaceNodeContent.bind(null, node));

  let document = node.ownerDocument;
  let isSvg = OWNER_SVG_ELEMENT in node;
  let upgrader = memoizeOnFirstArg(upgrade.bind(null, document, isSvg));

  return render.bind(null, upgrader, finalSideEffect)
};

const html = (strings, ...values) => {
  if (isArray(strings)) {
    return new TagInvocation(strings, values, false)
  }
  let key = strings;
  return (strings, ...values) =>
    new TagInvocation(strings, values, false, key)
};

const svg = (strings, ...values) => {
  if (isArray(strings)) {
    return new TagInvocation(strings, values, true)
  }
  let key = strings;
  return (strings, ...values) =>
    new TagInvocation(strings, values, true, key)
};

const materialize = tagInvocation => {
  return materializer()(tagInvocation)
};

const materializer = () => {
  let finalSideEffect = lruCacheOne(extractContent);
  let wire = lruCacheOne(isSvg =>
    lruCacheOne(upgrade.bind(null, document, isSvg))
  );
  return tagInvocation => {
    let { strings, values, isSvg } = tagInvocation;
    let upgrader = wire(isSvg);
    return render(upgrader, finalSideEffect, strings, ...values)
  }
};

const keyed = (key, tagInvocation) => {
  let { strings, values, isSvg } = tagInvocation;
  return new TagInvocation(strings, values, isSvg, key)
};

function wire(type = 'html') {
  switch (type) {
    case 'html':
      return wireHtml()
    case 'svg':
      return wireSvg()
  }
}

const wireHtml = () => {
  let finalSideEffect = lruCacheOne(extractContent);
  let upgrader = memoizeOnFirstArg(upgrade.bind(null, document, false));
  return render.bind(null, upgrader, finalSideEffect)
};

const wireSvg = () => {
  let finalSideEffect = lruCacheOne(extractContent);
  let upgrader = memoizeOnFirstArg(upgrade.bind(null, document, true));
  return render.bind(null, upgrader, finalSideEffect)
};

// - - - - - - - - - - - - - - - - - - - - - - -

// hyper.define('transformer', callback) 🌀
const define = (transformer, callback) => {
  if (!(transformer in transformers)) {
    transformersKeys.push(transformer);
  }
  transformers[transformer] = callback;
};

exports.bind = bind;
exports.html = html;
exports.svg = svg;
exports.materialize = materialize;
exports.materializer = materializer;
exports.keyed = keyed;
exports.wire = wire;
exports.wireHtml = wireHtml;
exports.wireSvg = wireSvg;
exports.define = define;
exports.repeat = repeat;

return exports;

}({}));
